package hu.uw.pallergabor.cts;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.widget.TextView;
import android.widget.Toast;

import java.util.UUID;

/**
 * Created by paller on 2014.12.26..
 */
public class CTSActivity extends Activity {
    public static String EXTRAS_DEVICE_NAME = "device_name";
    public static String EXTRAS_DEVICE_ADDRESS = "device_address";
    private Handler mHandler;
    private String deviceName;
    private String deviceAddress;
    private BluetoothAdapter bluetoothAdapter;
    private static final String LOG_TAG = "CTS.CTSActivity";
    public static final UUID CTS_SERVICE_UUID = UUID.fromString("00001805-0000-1000-8000-00805f9b34fb");
    public static final UUID CTS_CHARACTERISTIC_UUID = UUID.fromString("00002a2b-0000-1000-8000-00805f9b34fb");
    // 0x2902 org.bluetooth.descriptor.gatt.client_characteristic_configuration.xml
    public final static UUID CLIENT_CHARACTERISTIC_CONFIGURATION_UUID = UUID.fromString("00002902-0000-1000-8000-00805F9B34FB");
    private TextView currentDateTV;
    private TextView currentTimeTV;
    private TextView currentDayOfWeekTV;

    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_cts);
        final Intent intent = getIntent();
        deviceName = intent.getStringExtra(EXTRAS_DEVICE_NAME);
        deviceAddress = intent.getStringExtra(EXTRAS_DEVICE_ADDRESS);

        getActionBar().setTitle(deviceName);
        mHandler = new Handler();
        final BluetoothManager bm =
                (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        bluetoothAdapter = bm.getAdapter();
        currentDateTV = (TextView)findViewById(R.id.cts_current_date_tv);
        currentTimeTV = (TextView)findViewById(R.id.cts_current_time_tv);
        currentDayOfWeekTV = (TextView)findViewById(R.id.cts_current_dayofweek_tv);
    }

    @Override
    protected void onResume() {
        super.onResume();
        BluetoothDevice device = bluetoothAdapter.getRemoteDevice(deviceAddress);
        if( device != null ) {
            device.connectGatt(this,false,gattCallback);
            Log.d(LOG_TAG, "connectGatt");
        } else
            Log.d( LOG_TAG, "device==null");
    }

    public void onBackPressed () {
        if( btGatt == null ) {        // Finish normally if we don't have any connection
            super.onBackPressed();
        } else {                    // Else we start the disconnection and finish the activity only if the disconnection finished
            btGatt.disconnect();
            btGatt = null;
        }
    }

    private BluetoothGatt btGatt = null;
    private BluetoothGattService gattService;

    private void updateClock(byte ctdata[]) {
        if( ctdata.length >= 8 ) {
            final int year = (((int)ctdata[0]) & 0xFF ) | ((((int)ctdata[1]) << 8 ) & 0xFF00);
            final int month = ((int)ctdata[2]) & 0xFF;
            final int day = ((int)ctdata[3]) & 0xFF;
            final int hour = ((int)ctdata[4]) & 0xFF;
            final int min = ((int)ctdata[5]) & 0xFF;
            final int secs = ((int)ctdata[6]) & 0xFF;
            final int dayofweek = ((int)ctdata[7]) & 0xFF;
            mHandler.post(new Runnable(){
                final String days[] = {
                        "Sun","Mon","Tue","Wed","Thu","Fri","Sat"
                };

                public void run() {
                    String dateStr = String.format("%04d",year)+"-"+
                                    String.format("%02d",month)+"-"+
                                    String.format("%02d",day);
                    String timeStr = String.format("%02d",hour)+":"+
                                    String.format("%02d",min)+":"+
                                    String.format("%02d",secs);
                    String dayofweekStr = days[dayofweek-1];
                    currentDateTV.setText(dateStr);
                    currentTimeTV.setText(timeStr);
                    currentDayOfWeekTV.setText(dayofweekStr);
                }
            });
        }
    }

    private BluetoothGattCallback gattCallback = new BluetoothGattCallback() {
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {

            if( newState == BluetoothProfile.STATE_DISCONNECTED) {
                Log.d(LOG_TAG, "STATE_DISCONNECTED");
// The back button's default handling is disabled if we have a connection. By pressing back.
// the disconnection starts but we finish the activity if the disconnection is completed.
                finish();
            } else
            if( newState == BluetoothProfile.STATE_CONNECTED) {
                Log.d( LOG_TAG, "STATE_CONNECTED");
                btGatt = gatt;
                boolean success = gatt.discoverServices();
                Log.d( LOG_TAG, "discoverServices: "+success);
            }
        }

        // New services discovered
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                Log.d( LOG_TAG, "services discovered");
                gattService = gatt.getService(CTS_SERVICE_UUID);
                if( gattService == null ) {
                    Toast.makeText(CTSActivity.this, R.string.cts_not_supported, Toast.LENGTH_LONG).show();
                    finish();
                } else {
                    BluetoothGattCharacteristic ctsCharacteristic = gattService.getCharacteristic(CTS_CHARACTERISTIC_UUID);
                    gatt.setCharacteristicNotification(ctsCharacteristic, true);
                    BluetoothGattDescriptor ctsConfigDescriptor = ctsCharacteristic.getDescriptor(CLIENT_CHARACTERISTIC_CONFIGURATION_UUID);
                    if (ctsConfigDescriptor != null) {
                        ctsConfigDescriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
                        gatt.writeDescriptor(ctsConfigDescriptor);
                    } else {
                        Log.e(LOG_TAG, "Receive Characteristic can not be configured.");
                    }
                    int props = ctsCharacteristic.getProperties();
                    Log.d( LOG_TAG, "props: "+props);
                    boolean success = gatt.readCharacteristic(ctsCharacteristic);
                    Log.d( LOG_TAG,"read operation status: "+success);
                }
            } else {
                Log.w(LOG_TAG, "onServicesDiscovered received: " + status);
            }
        }

        public void onCharacteristicWrite (
                BluetoothGatt gatt,
                BluetoothGattCharacteristic characteristic,
                int status) {
            String success = status == BluetoothGatt.GATT_SUCCESS ?
                    "Success" :
                    "Failure";
            String message =
                    "Write operation on "+
                            characteristic.getUuid().toString()+
                            " : "+success;
            Log.d( LOG_TAG, "status: "+status+" : "+message);
        }

        public void onCharacteristicRead (BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            String message =
                    "Characteristic read: "+
                            characteristic.getUuid().toString();
            Log.d( LOG_TAG, message);
            byte v[] = characteristic.getValue();
            for( int i = 0 ; i < v.length ; ++i ) {
                Log.d( LOG_TAG, "v["+i+"]: "+v[i]);
            }
            updateClock(v);
        }

        public void onCharacteristicChanged (
                BluetoothGatt gatt,
                BluetoothGattCharacteristic characteristic) {
            String message =
                    "Characteristic changed: "+
                            characteristic.getUuid().toString();
            Log.d( LOG_TAG, message);
            byte v[] = characteristic.getValue();
            for( int i = 0 ; i < v.length ; ++i ) {
                Log.d( LOG_TAG, "v["+i+"]: "+v[i]);
            }
            updateClock(v);
        }
    };

}
